﻿using System;
using System.Collections.Generic;
using com.apifon.sdk;
using com.apifon.sdk.model;
using com.apifon.sdk.resource;
using com.apifon.sdk.response;
using apifon_sdk_csharp.com.apifon.sdk.model;
using apifon_sdk_csharp.com.apifon.sdk.resource;
using com.apifon.sdk.helpers;
using Newtonsoft.Json;
using apifon_sdk_csharp.com.apifon.sdk.helpers;

namespace apifon_sdk_csharp.com.apifon.Examples
{
    /**
     * This is a complete example on how to use the APIFON SDK for C#.
     */
    class MessagesExample
    {
        static void Main(string[] args)
        {
            /**
             * First things first, we need to set our credentials that we got from
             * the Mookee platform.
             */
            Mookee.AddCredentials("sms", "Your_SMS_Token", "Your_SMS_Secret_Key");
            Mookee.AddCredentials("viber", "Your_Viber_Token", "Your_Viber_Secret_Key");
            Mookee.AddCredentials("otp", "Your_OTP_Token", "Your_OTP_Secret_Key");

            int choice = 1;

            /**
             * These are our Resources.
             */
            SMSResource smsResource = new SMSResource();
            IMResource imResource = new IMResource();
            OTPResource otpResource = new OTPResource();

            /**
             * These are our Requests.
             */
            SmsRequest smsRequest = new SmsRequest();
            ImRequest imRequest = new ImRequest();
            OtpRequest otpRequest = new OtpRequest();
            RemoveRequest removeRequest = new RemoveRequest();

            /**
             * Here we generate the date that scheduled messages will be sent.
             */
            DateTime thisDate = new DateTime(2038, 6, 10, 15, 24, 16);
            string scheduledDate = thisDate.ToString("yyyy-MM-ddTH:mm:ss");

            /**
             * Here we create our subscribers.
             */
            string[] names = { "John", "George", "Alex", "Helen" };
            string[] names2 = { "Kate", "Julie", "Paul", "Mary" };
            List<string> numbers = new List<string>();
            numbers.Add("306999999999");
            //numbers.Add("306999999998");
            //numbers.Add("306999999997");

            /**
             * Here we define the content of our Request.
             */
            List<SubscriberInformation> subscribers;
            IMChannel imChannel;
            IMAction imAction;
            MessageContent msgContent;
            GatewayResponse response;
            OtpResponse otpResp;
            OtpVerifyResponse otpVerResp;
            string delResp;
            string callbackUrl = "https://yourserver/callback";
            string replyUrl = "https://yourserver/replyurl";
            String deletionType;

            while(choice != 0)
            {
                Console.WriteLine("------Menu------");
                Console.WriteLine("0.  Exit");
                Console.WriteLine("1.  Send Simple SMS");
                Console.WriteLine("2.  Send Complete SMS");
                Console.WriteLine("3.  Send Scheduled SMS");
                Console.WriteLine("4.  Remove Scheduled SMS");
                Console.WriteLine("5.  Send SMS With Landing Page");
                Console.WriteLine("6.  Case 1: Send IM With Text");
                Console.WriteLine("7.  Case 2: Send IM With Image");
                Console.WriteLine("8.  Case 3: Send IM With Text and Button");
                Console.WriteLine("9.  Case 4: Send IM With Text, Image and Button");
                Console.WriteLine("10. Send Complete IM without fallback");
                Console.WriteLine("11. Send Scheduled IM");
                Console.WriteLine("12. Remove Scheduled IM");
                Console.WriteLine("13. Send IM With Landing Page");
                Console.WriteLine("14. Send IM With Fallback SMS");
                Console.WriteLine("15. Send IM With Fallback Landing Page SMS");
                Console.WriteLine("16. Send 2-Way IM");
                Console.WriteLine("17. Send Balance Request");
                Console.WriteLine("18. Send Simple OTP Create Request");
                Console.WriteLine("19. Send Complete OTP Create Request");
                Console.WriteLine("20. Send OTP Verification Request");
                Console.Write("Choice: ");
                choice = Convert.ToInt32(Console.ReadLine());

                /**
                 * Every request has the same structure: 
                 *  *First we set the Credential we want to use. 
                 *  *Then we build the components of the Request. 
                 *  *Finally we build the Request and send it. 
                 *  *We print the response we get in console.
                 */
                switch (choice)
                {
                    case 0:
                        break;

                    /**
                     * This is a simple SMS Request.
                     */
                    case 1:
                        Mookee.SetActiveCredential("sms");

                        smsRequest = new SmsRequest();

                        msgContent = new MessageContent();
                        msgContent.Text = "Hello There!";
                        msgContent.SenderId = "Apifon";

                        smsRequest.AddStrSubscribers(numbers);
                        smsRequest.Message = msgContent;
                        smsRequest.CallbackUrl = callbackUrl;

                        Console.WriteLine("Sending Request to Endpoint: /services/sms/send");
                        Console.WriteLine("------ SMS Request Body ------");
                        PrintJson(smsRequest);
                        response = smsResource.Send(smsRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * In this SMS example we are taking advantage of every property that
                     * can be changed in a Request.
                     */
                    case 2:
                        Mookee.SetActiveCredential("sms");

                        smsRequest = new SmsRequest();

                        msgContent = new MessageContent();
                        msgContent.Text = "Hello {name}!";
                        msgContent.Dc = 0;
                        msgContent.SenderId = "Apifon";

                        subscribers = new List<SubscriberInformation>();
                        foreach(String num in numbers)
                        {
                            SubscriberInformation sub = new SubscriberInformation();
                            sub.Number = num;
                            sub.CustomId = "222";
                            sub.AddParams("name", names[subscribers.Count]);
                            subscribers.Add(sub);
                        }

                        smsRequest.AddSubscribers(subscribers);
                        smsRequest.ReferenceId = "reference";
                        smsRequest.Message = msgContent;
                        smsRequest.CallbackUrl = callbackUrl;
                        smsRequest.Tte = "86400";

                        Console.WriteLine("Sending Request to Endpoint: /services/sms/send");
                        Console.WriteLine("------ SMS Request Body ------");
                        PrintJson(smsRequest);
                        response = smsResource.Send(smsRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * Here we send a scheduled SMS Request.
                     */
                    case 3:
                        Mookee.SetActiveCredential("sms");

                        smsRequest = new SmsRequest();

                        msgContent = new MessageContent();
                        msgContent.Text = "Hello There!";
                        msgContent.SenderId = "Apifon";

                        smsRequest.AddStrSubscribers(numbers);
                        smsRequest.Message = msgContent;
                        smsRequest.CallbackUrl = callbackUrl;
                        smsRequest.Date = scheduledDate;

                        Console.WriteLine("Sending Request to Endpoint: /services/sms/send");
                        Console.WriteLine("------ SMS Request Body ------");
                        PrintJson(smsRequest);
                        response = smsResource.Send(smsRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * This is an SMS Deletion case.
                     */
                    case 4:
                        Mookee.SetActiveCredential("sms");

                        deletionType = "";
                        while (deletionType !="0" && deletionType != "1" && deletionType != "2")
                        {
                            Console.WriteLine("Do you want to delete scheduled messages using Request ID or Reference ID?\nEnter [0] for Request ID, [1] for Reference ID or [2] to exit: ");
                            deletionType = Console.ReadLine();
                            if (deletionType != "0" && deletionType != "1" && deletionType != "2")
                            {
                                Console.WriteLine("Invalid Option.");
                            }
                        }
                        if (deletionType != "2")
                        {
                            removeRequest = new RemoveRequest();
                            if (deletionType == "0")
                            {
                                removeRequest.Field = RemoveFieldType.REQUEST_ID.ToString();
                                Console.WriteLine("Please enter the Request ID of the scheduled message you want to delete: ");
                                removeRequest.Id = Console.ReadLine();
                            }
                            else
                            {
                                removeRequest.Field = RemoveFieldType.REFERENCE_ID.ToString();
                                Console.WriteLine("Please enter the Reference ID of the scheduled message you want to delete: ");
                                removeRequest.Id = Console.ReadLine();
                            }
                            Console.WriteLine("Sending Request to Endpoint: /services/sms/remove");
                            Console.WriteLine("------ SMS Remove Request Body ------");
                            PrintJson(removeRequest);
                            delResp = smsResource.Remove(removeRequest);
                            Console.WriteLine("------ SMS Remove Request Response ------\n" + delResp);
                        }
                        break;

                    /**
                     * In this example we are sending an SMS Request with a landing page.
                     * {apifon_lp} will later be replaced with the URL property of the
                     * landing page.
                     */
                    case 5:
                        Mookee.SetActiveCredential("sms");

                        smsRequest = new SmsRequest();

                        msgContent = new MessageContent();
                        msgContent.Text = "Hello {name}! {apifon_lp}";
                        msgContent.Dc = 0;
                        msgContent.SenderId = "Apifon";

                        subscribers = new List<SubscriberInformation>();
                        foreach (String num in numbers)
                        {
                            SubscriberInformation sub = new SubscriberInformation();
                            sub.Number = num;
                            sub.CustomId = "222";
                            sub.AddParams("name", names[subscribers.Count]);

                            LandingProperties lp = new LandingProperties();
                            lp.Url = "https://mookee.apifon.com/user/login/";
                            lp.AddData("name", names2[subscribers.Count]);
                            lp.Redirect = true;

                            sub.SetLandingPage(lp);
                            subscribers.Add(sub);
                        }

                        smsRequest.AddSubscribers(subscribers);
                        smsRequest.ReferenceId = "reference";
                        smsRequest.Message = msgContent;
                        smsRequest.CallbackUrl = callbackUrl;
                        smsRequest.Tte = "86400";

                        Console.WriteLine("Sending Request to Endpoint: /services/sms/send");
                        Console.WriteLine("------ SMS Request Body ------");
                        PrintJson(smsRequest);
                        response = smsResource.Send(smsRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * Viber Case 1: Send IM that contains only a text.
                     */
                    case 6:
                        Mookee.SetActiveCredential("viber");

                        imRequest = new ImRequest();

                        imChannel = new IMChannel();
                        imChannel.Text = "50% discount on our membership fee if you renew your subscription today!";
                        imChannel.SenderId = "Mookee";
                        
                        imRequest.AddStrSubscribers(numbers);
                        imRequest.CallbackUrl = callbackUrl;
                        imRequest.AddImChannel(imChannel);

                        Console.WriteLine("Sending Request to Endpoint: /services/im/send");
                        Console.WriteLine("------ IM Request Body ------");
                        PrintJson(imRequest);
                        response = imResource.Send(imRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * Viber Case 2: Send IM that contains only an image and define device target to smartphone only.
                     */
                    case 7:
                        Mookee.SetActiveCredential("viber");

                        imRequest = new ImRequest();

                        imChannel = new IMChannel();
                        imChannel.SenderId = "Mookee";
                        imChannel.AddImage("http://assets.apifon.com/images/boy_girl.jpg");
                        imChannel.DeviceTarget = DeviceTarget.SMARTPHONE_ONLY.ToString();

                        imRequest.AddStrSubscribers(numbers);
                        imRequest.CallbackUrl = callbackUrl;
                        imRequest.AddImChannel(imChannel);

                        Console.WriteLine("Sending Request to Endpoint: /services/im/send");
                        Console.WriteLine("------ IM Request Body ------");
                        PrintJson(imRequest);
                        response = imResource.Send(imRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * Viber Case 3: Send IM that contains text and action button.
                     */
                    case 8:
                        Mookee.SetActiveCredential("viber");

                        imRequest = new ImRequest();

                        imAction = new IMAction();
                        imAction.Title = "Renew now!";
                        imAction.TargetUrl = "https://www.apifon.com";

                        imChannel = new IMChannel();
                        imChannel.Text = "50% discount on our membership fee if you renew your subscription today!";
                        imChannel.SenderId = "Mookee";
                        imChannel.AddAction(imAction);

                        imRequest.AddStrSubscribers(numbers);
                        imRequest.CallbackUrl = callbackUrl;
                        imRequest.AddImChannel(imChannel);

                        Console.WriteLine("Sending Request to Endpoint: /services/im/send");
                        Console.WriteLine("------ IM Request Body ------");
                        PrintJson(imRequest);
                        response = imResource.Send(imRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * Viber Case 4: Send IM that contains text, image and action button.
                     */
                    case 9:
                        Mookee.SetActiveCredential("viber");

                        imRequest = new ImRequest();

                        imAction = new IMAction();
                        imAction.Title = "Renew now!";
                        imAction.TargetUrl = "https://www.apifon.com";

                        imChannel = new IMChannel();
                        imChannel.Text = "50% discount on our membership fee if you renew your subscription today!";
                        imChannel.SenderId = "Mookee";
                        imChannel.AddImage("http://assets.apifon.com/images/boy_girl.jpg");
                        imChannel.AddAction(imAction);

                        imRequest.AddStrSubscribers(numbers);
                        imRequest.CallbackUrl = callbackUrl;
                        imRequest.AddImChannel(imChannel);

                        Console.WriteLine("Sending Request to Endpoint: /services/im/send");
                        Console.WriteLine("------ IM Request Body ------");
                        PrintJson(imRequest);
                        response = imResource.Send(imRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * Complete Viber example.
                     */
                    case 10:
                        Mookee.SetActiveCredential("viber");

                        imRequest = new ImRequest();

                        imAction = new IMAction();
                        imAction.Title = "Renew now!";
                        imAction.TargetUrl = "https://www.apifon.com";

                        imChannel = new IMChannel();
                        imChannel.Id = "channelID";
                        imChannel.Text = "{name}! 50% discount on our membership fee if you renew your subscription today!";
                        imChannel.SenderId = "Mookee";
                        imChannel.AddImage("http://assets.apifon.com/images/boy_girl.jpg");
                        imChannel.AddAction(imAction);
                        imChannel.Ttl = 86400;
                        imChannel.ExpiryText = "Message Expired!";

                        subscribers = new List<SubscriberInformation>();
                        foreach (String num in numbers)
                        {
                            SubscriberInformation sub = new SubscriberInformation();
                            sub.Number = num;
                            sub.CustomId = "222";
                            sub.AddParams("name", names[subscribers.Count]);
                            subscribers.Add(sub);
                        }

                        imRequest.AddSubscribers(subscribers);
                        imRequest.ReferenceId = "reference";
                        imRequest.CallbackUrl = callbackUrl;
                        imRequest.AddImChannel(imChannel);

                        Console.WriteLine("Sending Request to Endpoint: /services/im/send");
                        Console.WriteLine("------ IM Request Body ------");
                        PrintJson(imRequest);
                        response = imResource.Send(imRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * Here we send a scheduled IM Request.
                     */
                    case 11:
                        Mookee.SetActiveCredential("viber");

                        imRequest = new ImRequest();

                        imAction = new IMAction();
                        imAction.Title = "Renew now!";
                        imAction.TargetUrl = "https://www.apifon.com";

                        imChannel = new IMChannel();
                        imChannel.Text = "50% discount on our membership fee if you renew your subscription today!";
                        imChannel.SenderId = "Mookee";
                        imChannel.AddImage("http://assets.apifon.com/images/boy_girl.jpg");
                        imChannel.AddAction(imAction);

                        imRequest.AddStrSubscribers(numbers);
                        imRequest.CallbackUrl = callbackUrl;
                        imRequest.AddImChannel(imChannel);
                        imRequest.Date = scheduledDate;

                        Console.WriteLine("Sending Request to Endpoint: /services/im/send");
                        Console.WriteLine("------ IM Request Body ------");
                        PrintJson(imRequest);
                        response = imResource.Send(imRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * This is an IM Deletion case.
                     */
                    case 12:
                        Mookee.SetActiveCredential("viber");

                        deletionType = "";
                        while (deletionType != "0" && deletionType != "1" && deletionType != "2")
                        {
                            Console.WriteLine("Do you want to delete scheduled messages using Request ID or Reference ID?\nEnter [0] for Request ID, [1] for Reference ID or [2] to exit: ");
                            deletionType = Console.ReadLine();
                            if (deletionType != "0" && deletionType != "1" && deletionType != "2")
                            {
                                Console.WriteLine("Invalid Option.");
                            }
                        }
                        if (deletionType != "2")
                        {
                            removeRequest = new RemoveRequest();
                            if (deletionType == "0")
                            {
                                removeRequest.Field = RemoveFieldType.REQUEST_ID.ToString();
                                Console.WriteLine("Please enter the Request ID of the scheduled message you want to delete: ");
                                removeRequest.Id = Console.ReadLine();
                            }
                            else
                            {
                                removeRequest.Field = RemoveFieldType.REFERENCE_ID.ToString();
                                Console.WriteLine("Please enter the Reference ID of the scheduled message you want to delete: ");
                                removeRequest.Id = Console.ReadLine();
                            }
                            Console.WriteLine("Sending Request to Endpoint: /services/im/remove");
                            Console.WriteLine("------ IM Remove Request Body ------");
                            PrintJson(removeRequest);
                            delResp = imResource.Remove(removeRequest);
                            Console.WriteLine("------ IM Remove Request Response ------\n" + delResp);
                        }
                        break;

                    /**
                     * In this example we send a Viber message containing a landing page.
                     */
                    case 13:
                        Mookee.SetActiveCredential("viber");

                        imRequest = new ImRequest();

                        imAction = new IMAction();
                        imAction.Title = "Renew now!";
                        imAction.TargetUrl = "{apifon_lp}";

                        imChannel = new IMChannel();
                        imChannel.Text = "{name}! 50% discount on our membership fee if you renew your subscription today!";
                        imChannel.SenderId = "Mookee";
                        imChannel.AddImage("http://assets.apifon.com/images/boy_girl.jpg");
                        imChannel.AddAction(imAction);

                        subscribers = new List<SubscriberInformation>();
                        foreach (String num in numbers)
                        {
                            SubscriberInformation sub = new SubscriberInformation();
                            sub.Number = num;
                            sub.CustomId = "222";
                            sub.AddParams("name", names[subscribers.Count]);

                            LandingProperties lp = new LandingProperties();
                            lp.Url = "https://mookee.apifon.com/user/login/";
                            lp.AddData("name", names2[subscribers.Count]);
                            lp.Redirect = true;

                            sub.SetLandingPage(lp);
                            subscribers.Add(sub);
                        }

                        imRequest.AddSubscribers(subscribers);
                        imRequest.CallbackUrl = callbackUrl;
                        imRequest.AddImChannel(imChannel);

                        Console.WriteLine("Sending Request to Endpoint: /services/im/send");
                        Console.WriteLine("------ IM Request Body ------");
                        PrintJson(imRequest);
                        response = imResource.Send(imRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * In this example we send a Viber message with a fallback SMS, in
                     * case Viber delivery fails.
                     */
                    case 14:
                        Mookee.SetActiveCredential("viber");

                        imRequest = new ImRequest();

                        imAction = new IMAction();
                        imAction.Title = "Renew now!";
                        imAction.TargetUrl = "https://www.apifon.com";

                        imChannel = new IMChannel();
                        imChannel.Text = "50% discount on our membership fee if you renew your subscription today!";
                        imChannel.SenderId = "Mookee";
                        imChannel.AddImage("http://assets.apifon.com/images/boy_girl.jpg");
                        imChannel.AddAction(imAction);

                        msgContent = new MessageContent();
                        msgContent.Text = "Hello There!";
                        msgContent.SenderId = "Apifon";

                        imRequest.AddStrSubscribers(numbers);
                        imRequest.MessageContent = msgContent;
                        imRequest.CallbackUrl = callbackUrl;
                        imRequest.AddImChannel(imChannel);

                        Console.WriteLine("Sending Request to Endpoint: /services/im/send");
                        Console.WriteLine("------ IM Request Body ------");
                        PrintJson(imRequest);
                        response = imResource.Send(imRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * In this example we send a Viber message with a landing page
                     * fallback SMS, in case Viber delivery fails.
                     */
                    case 15:
                        Mookee.SetActiveCredential("viber");

                        imRequest = new ImRequest();

                        imAction = new IMAction();
                        imAction.Title = "Renew now!";
                        imAction.TargetUrl = "https://www.apifon.com";

                        imChannel = new IMChannel();
                        imChannel.Text = "50% discount on our membership fee if you renew your subscription today!";
                        imChannel.SenderId = "Mookee";
                        imChannel.AddImage("http://assets.apifon.com/images/boy_girl.jpg");
                        imChannel.AddAction(imAction);

                        subscribers = new List<SubscriberInformation>();
                        foreach (String num in numbers)
                        {
                            SubscriberInformation sub = new SubscriberInformation();
                            sub.Number = num;
                            sub.CustomId = "222";
                            sub.AddParams("name", names[subscribers.Count]);

                            LandingProperties lp = new LandingProperties();
                            lp.Url = "https://mookee.apifon.com/user/login/";
                            lp.AddData("name", names2[subscribers.Count]);
                            lp.Redirect = true;

                            sub.SetLandingPage(lp);
                            subscribers.Add(sub);
                        }

                        msgContent = new MessageContent();
                        msgContent.Text = "Hello {name}! {apifon_lp}";
                        msgContent.SenderId = "Apifon";

                        imRequest.AddSubscribers(subscribers);
                        imRequest.MessageContent = msgContent;
                        imRequest.CallbackUrl = callbackUrl;
                        imRequest.AddImChannel(imChannel);

                        Console.WriteLine("Sending Request to Endpoint: /services/im/send");
                        Console.WriteLine("------ IM Request Body ------");
                        PrintJson(imRequest);
                        response = imResource.Send(imRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * This is a 2-way Viber example. Any replies sent by the 
                     * message recipient will generate a reply-message callback 
                     * on the replyURL. Note that this request would require use of 
                     * a Viber Sender ID with 2-way messaging enabled. 2-way 
                     * messaging can be enabled on a Sender ID via our platform.
                     */
                    case 16:
                        Mookee.SetActiveCredential("viber");

                        imRequest = new ImRequest();

                        imAction = new IMAction();
                        imAction.Title = "Renew now!";
                        imAction.TargetUrl = "https://www.apifon.com";

                        imChannel = new IMChannel();
                        imChannel.Text = "50% discount on our membership fee if you renew your subscription today! Would you like to renew your subscription?";
                        imChannel.SenderId = "Mookee";
                        imChannel.AddImage("http://assets.apifon.com/images/boy_girl.jpg");
                        imChannel.AddAction(imAction);

                        imRequest.AddStrSubscribers(numbers);
                        imRequest.CallbackUrl = callbackUrl;
                        imRequest.ReplyUrl = replyUrl;
                        imRequest.AddImChannel(imChannel);

                        Console.WriteLine("Sending Request to Endpoint: /services/im/send");
                        Console.WriteLine("------ IM Request Body ------");
                        PrintJson(imRequest);
                        response = imResource.Send(imRequest);
                        Console.WriteLine("------ Gateway Response Body ------");
                        PrintJson(response);
                        break;

                    /**
                     * This is a Balance Request example in which we are informed about our
                     * remaining Balance and our Plafon.
                     */
                    case 17:
                        Mookee.SetActiveCredential("sms");

                        BalanceResource balanceResource = new BalanceResource();

                        Console.WriteLine("Sending Request to Endpoint: /services/balance");
                        Console.WriteLine("------ Balance Request Body ------\nEmpty Body");
                        BalanceResponse balanceResp = balanceResource.Send();
                        Console.WriteLine("------ Balance Response Body ------");
                        PrintJson(balanceResp);
                        break;

                    /**
                     * This is a basic OTP Create Request example. We are only setting a phone
                     * number and a reference_id. Reference_id is important because it links to 
                     * the generated OTP.
                     */
                    case 18:
                        Mookee.SetActiveCredential("otp");

                        otpRequest = new OtpRequest();

                        Console.WriteLine("Enter a reference_id for your OTP create request: ");
                        otpRequest.ReferenceId = Console.ReadLine();
                        otpRequest.Subscriber = numbers[0];

                        Console.WriteLine("Sending Request to Endpoint: /services/otp/create");
                        Console.WriteLine("------ OTP Create Request Body ------");
                        PrintJson(otpRequest);
                        otpResp = otpResource.Create(otpRequest);
                        Console.WriteLine("------ OTP Create Response Body ------");
                        PrintJson(otpResp);
                        break;

                    /**
                     * This is a complete OTP Create Request example. We are specifying every
                     * field of the request to our preferences.
                     */
                    case 19:
                        Mookee.SetActiveCredential("otp");

                        otpRequest = new OtpRequest();

                        msgContent = new MessageContent();
                        msgContent.Text = "Hello There! Here is your OTP: {apifon_otp}";
                        msgContent.SenderId = "Apifon";

                        Console.WriteLine("Enter a reference_id for your OTP create request: ");
                        otpRequest.ReferenceId = Console.ReadLine();
                        otpRequest.Subscriber = numbers[0];
                        otpRequest.CodeLength = 6;
                        otpRequest.CodeType = "numeric";
                        otpRequest.CallbackUrl = callbackUrl;
                        otpRequest.Expire = 300;
                        otpRequest.Message = msgContent;

                        Console.WriteLine("Sending Request to Endpoint: /services/otp/create");
                        Console.WriteLine("------ OTP Create Request Body ------");
                        PrintJson(otpRequest);
                        otpResp = otpResource.Create(otpRequest);
                        Console.WriteLine("------ OTP Create Response Body ------");
                        PrintJson(otpResp);
                        break;

                    /**
                     * This is an OTP Verification Request example. When a subscriber is using
                     * the OTP that we generated for him, we want to verify that the OTP is valid.
                     * We are using the reference_id that we used to generate the OTP and the OTP that
                     * we are provided to make this verification.
                     */
                    case 20:
                        Mookee.SetActiveCredential("otp");

                        Console.WriteLine("Enter the reference_id of the OTP you want to verify: ");
                        String otpReference = Console.ReadLine();
                        Console.WriteLine("Enter the OTP you want to verify: ");
                        String otpCode = Console.ReadLine();

                        Console.WriteLine("Sending Request to Endpoint: /services/otp/verify/" + otpReference + "/" + otpCode);
                        Console.WriteLine("------ OTP Verify Request Body ------\nEmpty Body");
                        otpVerResp = otpResource.Verify(otpReference, otpCode);
                        Console.WriteLine("------ OTP Verify Response Body ------");
                        PrintJson(otpVerResp);
                        break;
                }
            }
        }

        public static void PrintJson(Object response)
        {
            String jsonString =  JsonConvert.SerializeObject(response);
            Console.WriteLine(JsonHelper.FormatJson(jsonString));
        }
    }
}
