﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using apifon_sdk_csharp.com.apifon.sdk.model;
using apifon_sdk_csharp.com.apifon.sdk.model.request;
using apifon_sdk_csharp.com.apifon.sdk.resource;
using apifon_sdk_csharp.com.apifon.sdk.resource.helpers;
using apifon_sdk_csharp.com.apifon.sdk.response;
using com.apifon.sdk;
using com.apifon.sdk.helpers;
using com.apifon.sdk.model.request;
using com.apifon.sdk.resource;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.Examples
{
    class SubscriberExample
    {
        static void Main(string[] args)
        {
            Mookee.AddCredentials("token", "Your_Token", "Your_Secret_Key");
            Mookee.SetActiveCredential("token");

            ListResource listResource = new ListResource();
            SubscriberFieldResource subscriberFieldResource = new SubscriberFieldResource();
            GdprFieldResource gdprFieldResource = new GdprFieldResource();
            SubscriberResource subscriberResource = new SubscriberResource();
            GroupResource groupResource = new GroupResource();
            SubgroupResource subgroupResource = new SubgroupResource();

            /**
                    * First you MUST create a List in order to add Subscribers in it.
            */
            // Create a Complete List
            CampaignDefaultsBuilder campaignDefaultsBuilder = CampaignDefaultsBuilder.getCampaignDefaultsBuilder();
            campaignDefaultsBuilder.setSenderId("Apifon").setSenderName("Apifon").setViberSenderId("Mookee").setFrom("apifon@apifon.com").setReplyTo("apifon@apifon.com");

            GdprPropertiesBuilder gdprPropertiesBuilder = GdprPropertiesBuilder.getGdprPropertiesBuilder(true);
            gdprPropertiesBuilder.setTitle("GDPR title").setLegalText("GDPR legal text").setDescription("GDPR description");

            ListRequestBuilder.ListRequest listRequest = ListRequestBuilder.getAddListRequestBuilder(ListType.ADVANCED.ToString(), "List from C#", "+30")
                .withCampaignDefaults(campaignDefaultsBuilder)
                .withGdprProperties(gdprPropertiesBuilder)
                .build();
            Console.WriteLine("Sending Request to Endpoint: /services/list/");
            Console.WriteLine("------ List Request Body ------");
            PrintJson(listRequest);
            Uri uri = listResource.createList(listRequest);
            Console.WriteLine(uri);

            //Get List with Uri
            ListResponse list = listResource.getByResourceUri(uri);
            PrintJson(list);

            // Add Subscriber Field to List
            SubscriberFieldRequestBuilder.SubscriberFieldRequest subscriberFieldRequest = SubscriberFieldRequestBuilder
                            .getAddRequestBuilder(list.getId(), SubscriberFieldType.TEXT.ToString(), 20, "First Name")
                            .build();
            Uri subscriberFieldUri = subscriberFieldResource.addSubscriberField(subscriberFieldRequest);
            Console.WriteLine(subscriberFieldUri);

            // Add GDPR field (Gpdr Compliant MUST be True)
            GdprFieldRequestBuilder.GdprFieldRequest gdprFieldRequest = GdprFieldRequestBuilder
                            .getAddGdprFieldRequestBuilder("Discount", list.getId()).build();
            Uri gdprFieldUri = gdprFieldResource.addGdprField(gdprFieldRequest);
            Console.WriteLine(gdprFieldUri);

            // Add group and subgroup to List
            GroupRequestBuilder.GroupRequest groupRequest = GroupRequestBuilder.getAddGroupRequestBuilder(list.getId(), GroupType.GROUP.ToString(), "my_group")
                .withSelectionType(SelectionType.MULTIPLE.ToString())
                .build();
            Uri groupUri = groupResource.addGroup(groupRequest);
            GroupResponse myGroup = groupResource.getByResourceUri(groupUri);

            SubgroupRequestBuilder.SubgroupRequest subgroupRequest = SubgroupRequestBuilder.getAddSubgroupRequestBuilder(list.getId(), myGroup.getField(), "my option").build();
            subgroupResource.addSubgroup(subgroupRequest);

            ListResponse updatedList = listResource.getByResourceUri(uri);
            List<SubgroupResponse> subgroupList = updatedList.getGroups()[0].getOptions();

            int choice = 1;

            //DUMMY uri for Initialization
            Uri subUri = new Uri("http://ars.apifon.com/globaluri");

            string listId = list.getId();

            while (choice != 0)
            {
                Console.WriteLine("------Menu------");
                Console.WriteLine("0.  Exit");
                Console.WriteLine("1.  Add a Subscriber to List");
                Console.WriteLine("2.  Get a Subscriber with a Uri");
                Console.WriteLine("3.  Get a Subscriber from List");
                Console.WriteLine("4.  Update a Subscriber to List");
                Console.WriteLine("5.  Get All Subscribers from List");
                Console.WriteLine("6.  Query Subscribers With Conditions");
                Console.WriteLine("7.  Subscribe a Subscriber");
                Console.WriteLine("8.  Unsubscribe a Subscriber");
                Console.WriteLine("9.  Remove a Subscriber from List");
                Console.Write("Choice: ");
                choice = Convert.ToInt32(Console.ReadLine());

                switch (choice)
                {
                    case 0:
                        break;

                    /**
                     * Add a Subscriber to List
                     * 
                     * Use SubscriberStatus enum class in order to send an acceptable string to API.
                     */
                    case 1:
                        SubscriberRequestBuilder subscriberRequest = SubscriberRequestBuilder.getAddSubscriberRequestBuilder(listId)
                            .withDestination("6999999999", SubscriberStatus.UNSUBSCRIBED.ToString())
                            .withEmail("apifon@apifon.com", SubscriberStatus.UNSUBSCRIBED.ToString())
                            .withSubscriberField("FIRST_NAME", "John")
                            .withGdprField("DISCOUNT", false);
                            //.withMobileGdprField("DISCOUNT", true) --> optional

                        List<String> optionFields = new List<String>();
                           foreach(SubgroupResponse option in subgroupList)
                        {
                            optionFields.Add(option.getField());
                        }

                        subscriberRequest.withGroupOptions(optionFields);
                        subUri = subscriberResource.addSubscriber(subscriberRequest.build()) ;
                        Console.WriteLine(subUri);

                        break;

                    /**
                    * Get a Subscriber from List
                    */
                    case 2:
                        SubscriberResponse sub = subscriberResource.getByResourceUri(subUri);
                        PrintJson(sub);

                        break;

                    /**
                    * Get a Subscriber from List
                    * 
                    * You need to get a SubscriberResponse first --> case 2
                    * 
                    * in order to have the destination or email Identifier
                    */
                    case 3:
                        //get a SubscriberResponse first --> case 2
                        SubscriberResponse sub2 = subscriberResource.getByResourceUri(subUri);

                        SubscriberSingleQuery subscriberSingleQuery = SubscriberSingleQuery.getSubscriberLookup(sub2.getDestinationId(), listId);
                        SubscriberResponse subscriberResponse = subscriberResource.getSubscriber(subscriberSingleQuery);
                        PrintJson(subscriberResponse);

                        break;

                    /**
                    * Update a Subscriber to List
                    * 
                    * * You need to get a SubscriberResponse first --> case 2
                    * 
                    * in order to have the destination or email Identifier
                    */
                    case 4:
                        //get a SubscriberResponse first --> case 2
                        SubscriberResponse subscriberResponse2 = subscriberResource.getByResourceUri(subUri);

                        SubscriberRequestBuilder.SubscriberRequest subscriberRequest1 = SubscriberRequestBuilder.getBuilderFromSubscriber(subscriberResponse2)
                            .clearSubscriberFields()
                            .withSubscriberField("FIRST_NAME", "Johnny")
                            .build();
                        subscriberResource.updateSubscriber(subscriberRequest1);

                        break;

                    /**
                    * Get All Subscribers from List using SubscriberQueryBuilder
                    * 
                    * Use SubscriberStatus enum class in order to send an acceptable string to API.
                    */
                    case 5:
                        PaginatedRequest paginatedRequest = SubscriberQueryBuilder
                            .getQueryBuilder(listId)
                            .withEmailStatus(SubscriberStatus.UNSUBSCRIBED.ToString()) //optional
                            .withStatus(SubscriberStatus.UNSUBSCRIBED.ToString()) //optional
                            .build();
                        CollectionEntityResponse<SubscriberResponse> collection = subscriberResource.querySubscribers(paginatedRequest);

                        PrintJson(collection);

                        break;

                    /**
                    * Query Subscribers With Conditions
                    * 
                    * Use ConditionOperator enum class in order to send an acceptable string to API.
                    */
                    case 6:
                        Condition condition = new Condition("destination", ConditionOperator.beginsWith.ToString(), "30699");
                        Sort sort = new Sort(SortDirection.ASC.ToString(), "destination");

                        SubscriberConditionQueryRequestBuilder.SubscriberConditionQueryRequest subscriberConditionQueryRequest = SubscriberConditionQueryRequestBuilder
                            .getSubscriberConditionsBuilder(listId)
                            .withCondition(condition)
                            .addVisibleField("destination") //optional
                            .withSort(sort) //optional
                            .build();
                        CollectionEntityResponse<SubscriberResponse> collection2 = subscriberResource.querySubscribersWithConditions(subscriberConditionQueryRequest);
                        PrintJson(collection2);

                        break;

                    /**
                    * Subscribe a Subscriber
                    * 
                    * * You need to get a SubscriberResponse first --> case 2
                    * 
                    * in order to have the destination or email Identifier
                    */
                    case 7:
                        //get a SubscriberResponse first --> case 2
                        SubscriberResponse subscriberResponse3 = subscriberResource.getByResourceUri(subUri);

                        subscriberResource.subscribeSubscriber(listId, subscriberResponse3.getDestinationId());

                        break;

                    /**
                    * Unsubscribe a Subscriber
                    * 
                    * * You need to get a SubscriberResponse first --> case 2
                    * 
                    * in order to have the destination or email Identifier
                    */
                    case 8:
                        //get a SubscriberResponse first --> case 2
                        SubscriberResponse subscriberResponse4 = subscriberResource.getByResourceUri(subUri);

                        subscriberResource.unsubscribeSubscriber(listId, subscriberResponse4.getDestinationId());

                        break;

                    /**
                    * Remove a Subscriber from List
                    * 
                    * * You need to get a SubscriberResponse first --> case 2
                    * 
                    * in order to have the destination or email Identifier
                    */
                    case 9:
                        //get a SubscriberResponse first --> case 2
                        SubscriberResponse subscriberResponse5 = subscriberResource.getByResourceUri(subUri);

                        subscriberResource.removeSubscriber(listId, subscriberResponse5.getDestinationId());

                        break;
                }
            }
        }

        public static void PrintJson(Object response)
        {
            String jsonString = JsonConvert.SerializeObject(response);
            Console.WriteLine(JsonHelper.FormatJson(jsonString));
        }
    }
}
