using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Security;
using System.Text;
using apifon_sdk_csharp.com.apifon.sdk;
using apifon_sdk_csharp.com.apifon.sdk.exception;
using apifon_sdk_csharp.com.apifon.sdk.resource.helpers;
using apifon_sdk_csharp.com.apifon.sdk.response;
using com.apifon.sdk.resource;
using com.apifon.utils;
using Newtonsoft.Json;
using static apifon_sdk_csharp.com.apifon.sdk.model.request.OAuthConfigurationBuilder;

namespace com.apifon.sdk
{
    public class Mookee
    {
        private static readonly string productionUrl = "http://ars.apifon.com";
        private static string stagingUrl = "https://ars-staging.apifon.com";
        private static readonly string IDENTITY_PRODUCTION_SERVER_URL = "https://ids.apifon.com/";
        private static volatile string IDENTITY_STAGING_SERVER_URL = "https://ids-staging.apifon.com/";

        private static string base_url = null;
        private static Dictionary<String, List<string>> credentials;

        private static bool staging = false;
        private static volatile Mookee instance = null;

        private static String activeOAuthIdentifier = null;
        private static bool autoRefreshTokens = false;

        private static string key;
        private static SignedRequestsHelper signer = null;

        //OAuthRegistry
        private static OAuthRegistry oAuthRegistryInstance = OAuthRegistry.GetInstance();

        private Mookee() {}

        public static Mookee GetInstance()
        {
            if(instance == null) {
                instance = new Mookee();
            }
            return instance;
        }

        public static void setMookeeToOAuthMode(string identifier, bool autoRefreshTokens)
        {
            if (isOAuthEnabled(identifier))
            {
                Mookee.activeOAuthIdentifier = identifier;
                Mookee.autoRefreshTokens = autoRefreshTokens;
                return;
            }
            throw new ArgumentException("Identifier " + identifier + " is not a valid OAuth identifier.");
        }

        public static void setMookeeToHMACTokenMode()
        {
            activeOAuthIdentifier = null;
        }

        public static void setStaging(bool enable, string host, string identityServerHost)
        {
            staging = enable;
            if (host != null)
            {
                stagingUrl = host;
            }
            if (identityServerHost != null)
            {
                IDENTITY_STAGING_SERVER_URL = identityServerHost;
            }
        }

        public static void SetStaging(bool enable, string host)
        {
            Mookee.setStaging(enable, host, null);
        }

        public static void addOAuthResource(string identifier, OAuthConfiguration oauthConfiguration)
        {
            oAuthRegistryInstance.addOAuthResource(identifier, oauthConfiguration);
        }

        public static void removeOAuthResource(string identifier, bool revokeTokens)
        {
            oAuthRegistryInstance.removeOAuthResource(identifier, revokeTokens);
        }

        public static void revokeTokens(string identifier)
        {
            oAuthRegistryInstance.revokeTokens(identifier);
        }

        public static bool isOAuthEnabled(string identifier)
        {
            return oAuthRegistryInstance.isOAuthEnabled(identifier);
        }

        public static void retrieveClientCredentialsToken(string identifier)
        {
            oAuthRegistryInstance.retrieveTokensForClientCredential(identifier);
        }

        public static void retrieveClientCredentialsAndGetToken(string identifier)
        {
            oAuthRegistryInstance.retrieveTokensForClientCredentialAndGet(identifier);
        }

        public static void refreshAccessToken(string identifier)
        {
            oAuthRegistryInstance.refreshTokens(identifier);
        }

        public static String Dispatch(AbstractResource resource)
        {
            if (staging)
            {
                base_url = stagingUrl;
            }
            else
            {
                base_url = productionUrl;
            }

            string endpoint = resource.GetCurEndpoint();
            string body = resource.GetCurBody();
            string method = resource.GetCurMethod();

            try {
                return Mookee.MakeCall(endpoint, method, body);
            } catch (Exception ex) {
                throw ex;
            }

        }

        public static WebResponse DispatchToResponse(AbstractResource resource)
        {
            if (staging)
            {
                base_url = stagingUrl;
            }
            else
            {
                base_url = productionUrl;
            }

            string endpoint = resource.GetCurEndpoint();
            string body = resource.GetCurBody();
            string method = resource.GetCurMethod();

           try
           {
                Dictionary<string, string> headers = new Dictionary<string, string>();
                resource.addHeaders(headers);

                if (Mookee.activeOAuthIdentifier != null)
                {
                    return Mookee.makeOauthCallToResponse(endpoint, method, body, headers);
                }
                else 
                {
                    return Mookee.MakeCalltoResponse(endpoint, method, body, headers);
                }
                    
            }
            catch (WebException ex)
            {
                var response = ex.Response as HttpWebResponse;
                if (response.StatusCode == HttpStatusCode.NotModified)
                { 
                    return response;
                }
                else 
                {
                    string responseContent = getResponseBody(ex.Response);
                    ApiErrorResponse resp = JsonConvert.DeserializeObject<ApiErrorResponse>(responseContent);
                    ApifonRestException apifonRestException = new ApifonRestException(resp);
                    Console.WriteLine(apifonRestException.ToString());
                    throw apifonRestException;
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private static string MakeCall(string endpoint, string method, string body)
        {
            try
            {
                Dictionary<string, string> headers = new Dictionary<string, string>();
                WebResponse response;
                if (Mookee.activeOAuthIdentifier != null)
                {
                    response = makeOauthCallToResponse(endpoint, method, body, headers);
                }
                else
                {
                    response = MakeCalltoResponse(endpoint, method, body, headers);
                }
                return getResponseBody(response);
            }
            catch (WebException ex)
            {
                var response = ex.Response as HttpWebResponse;
                if (response.StatusCode == HttpStatusCode.Unauthorized)
                {
                    throw new SecurityException("401 Unauthorized");
                }
                else
                {
                    //For Status Code equals to 403
                    return getResponseBody(response);
                }

            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        private static WebResponse MakeCalltoResponse(string endpoint, string method, string body, Dictionary<string, string> headers)
        {
         
            if (body == null)
            {
                body = "";
            }

            String uri;
            if (endpoint.StartsWith(base_url))
            {
                uri = endpoint;
            }
            else
            {
                uri = base_url + endpoint;
            }

            signer = new SignedRequestsHelper(credentials[key][1]);
            String strDate = DateTime.Now.ToUniversalTime().ToString("r");
            Uri tmpUri = new Uri(uri);
            byte[] data = Encoding.UTF8.GetBytes(body);

            WebRequest webRequest = WebRequest.Create(uri);

            webRequest.Method = method;
            webRequest.ContentType = "application/json";
            webRequest.ContentLength = data.Length;
            string toSign = method + "\n"
                + tmpUri.AbsolutePath + "\n"
                + body + "\n"
                + strDate;
            string signature = signer.hmac(toSign);
            webRequest.Headers["X-ApifonWS-Date"] = strDate;
            webRequest.Headers["Authorization"] = "ApifonWS " + credentials[key][0] + ":" + signature;

            //Add the headers from AbstractResource
            if (headers.Count!=0) 
            {
                foreach (KeyValuePair<string, string> entry in headers)
                {
                    webRequest.Headers.Add(entry.Key, entry.Value);
                }
            }

            

            //If method equals to "GET" then do NOT write a body.
            if (!method.Equals("GET"))
            {
                using (Stream stream = webRequest.GetRequestStream())
                {
                    stream.Write(data, 0, data.Length);
                }
            }

            WebResponse response = webRequest.GetResponse();
            return response;
        }

        public static void AddCredentials(string key, string tkn, string prKey)
        {
            if (Mookee.credentials == null)
            {
                Mookee.credentials = new Dictionary<string, List<string>>();
            }
            var list = new List<string>();
            list.Add(tkn);
            list.Add(prKey);
            Mookee.credentials.Add(key, list);
        }

        public static void SetActiveCredential(string k)
        {
            Mookee.key = k;
        }

        public static String getBaseUrl()
        {
            return base_url;
        }

        private static WebResponse makeOauthCallToResponse(string endpoint, string method, string body, Dictionary<string, string> headers)
        {
            if (body == null)
            {
                body = "";
            }

            String uri;
            if (endpoint.StartsWith(base_url))
            {
                uri = endpoint;
            }
            else
            {
                uri = base_url + endpoint;
            }

            
            byte[] data = Encoding.UTF8.GetBytes(body);

            WebRequest webRequest = WebRequest.Create(uri);

            webRequest.Method = method;
            webRequest.ContentType = "application/json";
            webRequest.ContentLength = data.Length;
            webRequest.Headers["Authorization"] = "Bearer " + getActiveOAuthCredential();

            //Add the headers from AbstractResource
            if (headers.Count != 0)
            {
                foreach (KeyValuePair<string, string> entry in headers)
                {
                    webRequest.Headers.Add(entry.Key, entry.Value);
                }
            }

            //If method equals to "GET" then do NOT write a body.
            if (!method.Equals("GET"))
            {
                using (Stream stream = webRequest.GetRequestStream())
                {
                    stream.Write(data, 0, data.Length);
                }
            }

            try
            {
                HttpWebResponse response = (HttpWebResponse)webRequest.GetResponse();
                return response;
            }
            catch(WebException ex)
            {
                var resp = ex.Response as HttpWebResponse;
                // Unauthorized to access API, check auto refresh and try to refresh token manually from IDS.
                if (resp.StatusCode == HttpStatusCode.Unauthorized)
                {
                    if (Mookee.autoRefreshTokens)
                    {
                        // Try to refresh tokens and resubmit request!
                        try
                        {
                            oAuthRegistryInstance.refreshTokens(Mookee.activeOAuthIdentifier);
                            webRequest.Headers["Authorization"] = "Bearer " + getActiveOAuthCredential();

                            try
                            {
                                return (HttpWebResponse)webRequest.GetResponse();
                            }
                            catch (WebException ex1)
                            {
                                var resp1 = ex1.Response as HttpWebResponse;
                                if (resp1.StatusCode == HttpStatusCode.Unauthorized)
                                {
                                    throw new OAuthTokenException("Status: 401" + Environment.NewLine + "Error: " + resp1.Headers.Get("WWW-Authenticate"));
                                }
                            } 
                        }
                        catch (OAuthException oAuthException)
                        {
                            throw oAuthException;
                        }
                    }
                    else
                    {
                        throw new OAuthException("Unauthorized to access API");
                    }
                }
                else
                {
                    throw new WebException("",ex,ex.Status,ex.Response);
                }
            }
            return null;
        }

        private static String getActiveOAuthCredential()
        {
            OAuthCredentials oauthCredentials = oAuthRegistryInstance.retrieveActiveTokens(Mookee.activeOAuthIdentifier);
            if (oauthCredentials.hasExpired() && Mookee.autoRefreshTokens) 
            {
                oAuthRegistryInstance.refreshTokens(Mookee.activeOAuthIdentifier);
                return oAuthRegistryInstance.retrieveActiveTokens(Mookee.activeOAuthIdentifier).getAccessToken();
            }
            return oauthCredentials.getAccessToken();
        }

        public string getIdentityServerUrl()
        {
            if (Mookee.staging)
            {
                return Mookee.IDENTITY_STAGING_SERVER_URL;
            }
            return Mookee.IDENTITY_PRODUCTION_SERVER_URL;
        }

        public static string getResponseBody(WebResponse response)
        {
            string responseContent = null;
            using (Stream stream = response.GetResponseStream())
            {
                using (StreamReader sr99 = new StreamReader(stream))
                {
                    responseContent = sr99.ReadToEnd();
                }
            }
            return responseContent;
        }

    }
    
}