﻿using apifon_sdk_csharp.com.apifon.sdk.exception;
using apifon_sdk_csharp.com.apifon.sdk.model.request;
using apifon_sdk_csharp.com.apifon.sdk.resource;
using apifon_sdk_csharp.com.apifon.sdk.resource.helpers;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.sdk
{
    public class OAuthRegistry
    {
        private static volatile OAuthRegistry instance = null;

        private readonly Dictionary<string, OAuthResource> oauthRegistry;

        private OAuthRegistry()
        {
            oauthRegistry = new Dictionary<string, OAuthResource>();
        }

        public static OAuthRegistry GetInstance()
        {
            if (instance == null)
            {
                instance = new OAuthRegistry();
            }
            return instance;
        }

        public void addOAuthResource(string identifier, OAuthConfigurationBuilder.OAuthConfiguration oauthConfiguration)
        {
            OAuthResource oAuthResource = new OAuthResource(oauthConfiguration);
            if (oauthRegistry.ContainsKey(identifier)) 
            {
                removeOAuthResource(identifier, true);
            }
            oauthRegistry.Add(identifier, oAuthResource);
        }

        public void removeOAuthResource(string identifier, bool revoketokens)
        {
            if (oauthRegistry.ContainsKey(identifier)) 
            {
                if (revoketokens) 
                {
                    revokeTokens(identifier);
                }
                oauthRegistry.Remove(identifier);
            }
        }

        public bool isOAuthEnabled(string identifier)
        {
            return oauthRegistry.ContainsKey(identifier);
        }

        public void revokeTokens(string identifier) 
        {
            validateOAuthEnabled(identifier);
            oauthRegistry[identifier].revokeTokens();
        }

        public OAuthResource getResourceForIdentifier(string identifier) 
        {
            validateOAuthEnabled(identifier);
            return oauthRegistry[identifier];
        }

        public void retrieveTokensForClientCredential(string identifier)
        {
            validateOAuthEnabled(identifier);
            OAuthResource resource = oauthRegistry[identifier];
            resource.getTokensForClientCredentialsGrant();
        }

        public OAuthCredentials retrieveTokensForClientCredentialAndGet(string identifier)
        {
            validateOAuthEnabled(identifier);
            OAuthResource resource = oauthRegistry[identifier];
            resource.getTokensForClientCredentialsGrant();
            try 
            {
                return (OAuthCredentials)resource.getOauthCredentials().Clone();
            } 
            catch (Exception ex) 
            {
                throw new OAuthException(ex);
            }
        }

        public void refreshTokens(string identifier)
        {
            validateOAuthEnabled(identifier);
            oauthRegistry[identifier].refreshToken();
        }

        public OAuthCredentials retrieveActiveTokens(string identifier)
        {
            if (isOAuthEnabled(identifier)) 
            {
                try 
                {
                    return (OAuthCredentials)getResourceForIdentifier(identifier).getOauthCredentials().Clone();
                } 
                catch (Exception ex) 
                {
                    throw new OAuthException(ex);
                }
                
            }
            throw new OAuthException("Active tokens for " + identifier + " not located.");
        }

        private void validateOAuthEnabled(string identifier)
        {
            if (!isOAuthEnabled(identifier)) 
            {
                throw new OAuthException("Identifier " + identifier + " doesn't have OAuth enabled.");
            }
        }
    }
}
