﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    public class BounceQueryBuilder : PaginatedRequestBuilder, IFieldSerializable<BounceQueryBuilder>
    {
        private string type;
        private string reason;
        private readonly HashSet<string> fields;

        private BounceQueryBuilder()
        {
            this.fields = new HashSet<string>();
        }

        public static BounceQueryBuilder getBuilder()
        {
            return new BounceQueryBuilder();
        }

        public BounceQueryBuilder withType(string type)
        {
            if (string.IsNullOrEmpty(type)) { throw new System.ArgumentNullException("Parameter type is null."); }
            this.type = type;
            return this;
        }

        public BounceQueryBuilder withReason(string reason)
        {
            if (string.IsNullOrEmpty(reason)) { throw new System.ArgumentNullException("Parameter reason is null."); }
            this.reason = reason;
            return this;
        }

        public override PaginatedRequest build()
        {
            return new QueryBounceSubscribersRequest(this);
        }

        public override string toQueryParamRequest()
        {
            return new QueryBounceSubscribersRequest(this).toQueryParamRequest();
        }

        public HashSet<string> getFields()
        {
            return this.fields;
        }

        public BounceQueryBuilder addVisibleFields(Collection<string> fields)
        {
            if (fields == null) { throw new System.ArgumentNullException("Parameter fields is null."); }
            this.getFields().UnionWith(fields);
            return this;
        }

        public BounceQueryBuilder addVisibleField(string field)
        {
            if (field == null) { throw new System.ArgumentNullException("Parameter field is null."); }
            this.getFields().Add(field);
            return this;
        }

        public override string getPathId()
        {
            throw new NotImplementedException();
        }

        public string getQueryParams()
        {
            throw new NotImplementedException();
        }

        public class QueryBounceSubscribersRequest : PaginatedRequest
        {
            private readonly string type;
            private readonly string reason;
            private readonly HashSet<String> fields;

            public QueryBounceSubscribersRequest(BounceQueryBuilder builder)
            {
                this.type = builder.type;
                this.reason = builder.reason;
                this.fields = builder.fields;
                this.size = builder.size;
                this.page = builder.page;
                this.sortField = builder.sortField;
                this.sortDir = builder.sortDir;
            }
            
            public override string toQueryParamRequest()
            {
                List<KeyValuePair<string, string>> queryParams = new List<KeyValuePair<string, string>>();
                addQueryParams(queryParams);
                if (!string.IsNullOrEmpty(type))
                {
                    queryParams.Add(new KeyValuePair<string, string>("type", type));
                }
                if (!string.IsNullOrEmpty(reason))
                {
                    queryParams.Add(new KeyValuePair<string, string>("reason", reason));
                }
                if (fields != null && fields.Any(x => true))
                {
                    string join = string.Join(",", fields);
                    queryParams.Add(new KeyValuePair<string, string>("fields", join));
                }

                string joinParams = "";
                foreach (KeyValuePair<string, string> entry in queryParams)
                {
                    if (joinParams.Equals(""))
                    {
                        //For the first time
                        joinParams = entry.Key + "=" + entry.Value;
                    }
                    else
                    {
                        //Add "&" in query
                        joinParams = joinParams + "&" + entry.Key + "=" + entry.Value;
                    }
                }

                UriBuilder uriBuilder = new UriBuilder();
                uriBuilder.Query = joinParams;
                return uriBuilder.Query;
            }

            public override string getPathId()
            {
                throw new NotImplementedException();
            }
        }
    }
}
