﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    public class GdprFieldRequestBuilder
    {
        private readonly string listId;
        private string field;
        private string name;
        private string description;
        private string etag;

        private GdprFieldRequestBuilder(string name, string listId)
        {
            this.name = name;
            this.listId = listId;
        }

        public static GdprFieldRequestBuilder getUpdateGdprFieldRequestBuilder(string name, string listId, string field)
        {
            return new GdprFieldRequestBuilder(name, listId).withField(field);
        }

        public static GdprFieldRequestBuilder getAddGdprFieldRequestBuilder(string name, string listId)
        {
            return new GdprFieldRequestBuilder(name, listId);
        }

        public GdprFieldRequestBuilder withDescription(string description)
        {
            this.description = description;
            return this;
        }

        public GdprFieldRequestBuilder withEtag(string etag)
        {
            this.etag = etag;
            return this;
        }


        public GdprFieldRequestBuilder withName(string name)
        {
            this.name = name;
            return this;
        }

        public GdprFieldRequestBuilder withField(string field)
        {
            this.field = field;
            return this;
        }

        public GdprFieldRequest build()
        {
            return new GdprFieldRequest(this);
        }

        [DataContract]
        public class GdprFieldRequest : IJsonSerializable, IHttpEtag
        {
            [ScriptIgnore]
            private readonly string etag;
            [ScriptIgnore]
            private readonly string listId;
            [ScriptIgnore]
            private readonly string field;
            [DataMember(Name = "name")]
            private readonly string name;
            [DataMember(Name = "description")]
            private readonly string description;

            public GdprFieldRequest(GdprFieldRequestBuilder builder)
            {
                this.name = builder.name;
                this.description = builder.description;
                this.etag = builder.etag;
                this.listId = builder.listId;
                this.field = builder.field;
            }

            public string getName()
            {
                return name;
            }

            public string getDescription()
            {
                return description;
            }

            public string getListId()
            {
                return listId;
            }

            public string getField()
            {
                return field;
            }

            public string getBody()
            {
                return JsonConvert.SerializeObject(this, Formatting.Indented,
                            new JsonSerializerSettings
                            {
                                NullValueHandling = NullValueHandling.Ignore
                            });
            }

            public string getEtag()
            {
                return this.etag;
            }
        }
    }
}
