﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace com.apifon.sdk.model.request
{
    
    public class GdprPropertiesBuilder
    {
        private readonly bool gdprCompliant;
        private string title;
        private string description;
        private string legalText;

        private GdprPropertiesBuilder(bool gdprCompliant)
        {
            this.gdprCompliant = gdprCompliant;
        }

        private GdprPropertiesBuilder(GdprProperties gdprProperties)
        {
            this.gdprCompliant = gdprProperties.isGdprCompliant();
            this.description = gdprProperties.getDescription();
            this.legalText = gdprProperties.getLegalText();
            this.title = gdprProperties.getTitle();
        }

        public static GdprPropertiesBuilder getBuilderFromGdprProperties(GdprProperties gdprProperties)
        {
            return new GdprPropertiesBuilder(gdprProperties);
        }

        public static GdprPropertiesBuilder getGdprPropertiesBuilder(bool gdprCompliant)
        {
            return new GdprPropertiesBuilder(gdprCompliant);
        }

        public GdprPropertiesBuilder setTitle(string title)
        {
            this.title = title;
            return this;
        }

        public GdprPropertiesBuilder setDescription(string description)
        {
            this.description = description;
            return this;
        }

        public GdprPropertiesBuilder setLegalText(string legalText)
        {
            this.legalText = legalText;
            return this;
        }

        public GdprProperties build()
        {
            return new GdprProperties(this);
        }

        [DataContract]
        public class GdprProperties
        {
            [DataMember(Name = "gdpr_compliant")]
            private readonly bool gdprCompliant;
            [DataMember(Name = "title")]
            private readonly string title;
            [DataMember(Name = "description")]
            private readonly string description;
            [DataMember(Name = "legal_text")]
            private readonly string legalText;

            public GdprProperties()
            {
               
            }

            public GdprProperties(GdprPropertiesBuilder builder)
            {
                this.gdprCompliant = builder.gdprCompliant;
                this.title = builder.title;
                this.description = builder.description;
                this.legalText = builder.legalText;
            }

            public bool isGdprCompliant()
            {
                return gdprCompliant;
            }

            public string getTitle()
            {
                return title;
            }

            public string getDescription()
            {
                return description;
            }

            public string getLegalText()
            {
                return legalText;
            }
        }
    }
}
