﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using apifon_sdk_csharp.com.apifon.sdk.response;
using com.apifon.sdk.helpers;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace com.apifon.sdk.model.request
{
    public class ListRequestBuilder : IHttpEtag
    {
        [ScriptIgnore]
        private readonly string id;
        private readonly string type;
        private string title;
        private string prefix;
        private GdprPropertiesBuilder.GdprProperties gdprProperties;
        private CampaignDefaultsBuilder.CampaignDefaults campaignDefaults;
        private string description;
        private string etag;

        private ListRequestBuilder(string type, string title, string prefix, string listId)
        {
            this.type = type;
            this.title = title;
            this.prefix = prefix;
            this.id = listId;
        }

        private ListRequestBuilder(string listId)
        {
            this.type = null;
            this.id = listId;
        }

        public static ListRequestBuilder getUpdateListRequestBuilder(ListResponse list)
        {
            if (list == null) { throw new System.ArgumentNullException("Parameter list is null."); }
            if (string.IsNullOrEmpty(list.getId())) { throw new System.ArgumentNullException("Parameter list.id is null."); }

            return new ListRequestBuilder(null, list.getTitle(), list.getPrefix(), list.getId())
                .withCampaignDefaults(list.getCampaignDefaults())
                .withGdprProperties(list.getGdprProperties())
                .withDescription(list.getDescription())
                .withEtag(list.getEtag());
        }

        public static ListRequestBuilder getUpdateListRequestBuilder(string listId)
        {
            if (string.IsNullOrEmpty(listId)) { throw new System.ArgumentNullException("Parameter listId is null."); }
            return new ListRequestBuilder(listId);
        }

        public static ListRequestBuilder getAddListRequestBuilder(string type, string title, string prefix)
        {
            if (string.IsNullOrEmpty(type)) { throw new System.ArgumentNullException("Parameter type is null."); }
            if (string.IsNullOrEmpty(title)) { throw new System.ArgumentNullException("Parameter title is null."); }
            if (string.IsNullOrEmpty(prefix)) { throw new System.ArgumentNullException("Parameter prefix is null."); }
            return new ListRequestBuilder(type, title, prefix, null);
        }

        public ListRequestBuilder withTitle(string title)
        {
            if (string.IsNullOrEmpty(title)) { throw new System.ArgumentNullException("Parameter title is null."); }
            this.title = title;
            return this;
        }

        public ListRequestBuilder withPrefix(string prefix)
        {
            if (string.IsNullOrEmpty(prefix)) { throw new System.ArgumentNullException("Parameter prefix is null."); }
            this.prefix = prefix;
            return this;
        }

        
        public ListRequestBuilder withGdprProperties(GdprPropertiesBuilder builder)
        {
            this.gdprProperties = builder.build();
            return this;
        }

        
        public ListRequestBuilder withGdprProperties(GdprPropertiesBuilder.GdprProperties gdprProperties)
        {
            this.gdprProperties = gdprProperties;
            return this;
        }

        
        public ListRequestBuilder withCampaignDefaults(CampaignDefaultsBuilder builder)
        {
            this.campaignDefaults = builder.build();
            return this;
        }

       
        public ListRequestBuilder withCampaignDefaults(CampaignDefaultsBuilder.CampaignDefaults campaignDefaults)
        {
            this.campaignDefaults = campaignDefaults;
            return this;
        }

        
        public ListRequestBuilder withDescription(string description)
        {
            this.description = description;
            return this;
        }
       
        public ListRequestBuilder withEtag(string etag)
        {
            this.etag = etag;
            return this;
        }

        public ListRequest build()
        {
            return new ListRequest(this);
        }

        public string getEtag()
        {
            return this.etag;
        }

        [DataContract]
        public class ListRequest : IJsonSerializable , IHttpEtag
        {
            [ScriptIgnore]
            private readonly string id;
            [ScriptIgnore]
            private readonly string etag;
            [DataMember(Name = "type")]
            private readonly string type;
            [DataMember(Name = "title")]
            private readonly string title;
            [DataMember(Name = "prefix")]
            private readonly string prefix;
            [DataMember(Name = "gdpr_properties")]
            private readonly GdprPropertiesBuilder.GdprProperties gdprProperties;
            [DataMember(Name = "campaign_defaults")]
            private readonly CampaignDefaultsBuilder.CampaignDefaults campaignDefaults;
            [DataMember(Name = "description")]
            private readonly string description;
            
            public ListRequest(ListRequestBuilder builder)
            {
                this.gdprProperties = builder.gdprProperties;
                this.campaignDefaults = builder.campaignDefaults;
                this.type = builder.type;
                this.title = builder.title;
                this.prefix = builder.prefix;
                this.description = builder.description;
                this.id = builder.id;
                this.etag = builder.getEtag();
            }

            public GdprPropertiesBuilder.GdprProperties getGdprProperties()
            {
                return gdprProperties;
            }

            public CampaignDefaultsBuilder.CampaignDefaults getCampaignDefaults()
            {
                return campaignDefaults;
            }

            public string getType()
            {
                return type;
            }

            public string getTitle()
            {
                return title;
            }

            public string getPrefix()
            {
                return prefix;
            }

            public string getDescription()
            {
                return description;
            }

            public string getId()
            {
                return id;
            }

            public string getBody()
            {
                return JsonConvert.SerializeObject(this, Formatting.Indented,
                            new JsonSerializerSettings
                            {
                                NullValueHandling = NullValueHandling.Ignore
                            });
            }

            public string getEtag()
            {
                return this.etag;
            }
        }
    }
}
