﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    public class ListSingleQuery : IFieldSerializable<ListSingleQuery>, IHttpEtag
    {

        private readonly string listId;
        private readonly HashSet<string> fields;
        private string etag;

        private ListSingleQuery(string listId)
        {
            this.listId = listId;
            this.fields = new HashSet<string>();
        }

        public static ListSingleQuery getListRequest(string listId)
        {
            if (string.IsNullOrEmpty(listId)) { throw new System.ArgumentNullException("Parameter listId is null."); }
            return new ListSingleQuery(listId);
        }

        public ListSingleQuery withEtag(string etag)
        {
            this.etag = etag;
            return this;
        }

        public HashSet<string> getFields()
        {
            return this.fields;
        }

        public string getPathId()
        {
            return this.listId;
        }

        public string getEtag()
        {
            return this.etag;
        }

        public ListSingleQuery addVisibleFields(Collection<string> fields)
        {
            if (fields == null) { throw new System.ArgumentNullException("Parameter fields is null."); }
            this.getFields().UnionWith(fields);
            return this;
        }

        public ListSingleQuery addVisibleField(string field)
        {
            if (field == null) { throw new System.ArgumentNullException("Parameter field is null."); }
            this.getFields().Add(field);
            return this;
        }

        public string getQueryParams()
        {
            string joinFields = "";
            if ( getFields() != null && getFields().Any(x => true))
            {
                joinFields = string.Join(",", getFields());

                UriBuilder uriBuilder = new UriBuilder();
                uriBuilder.Query = "fields=" + joinFields;
                return uriBuilder.Query;
            }
            else
            {
                return "";
            } 
        }
                
    }
}
