﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    class SubgroupRequestBuilder
    {
        private  string listId;
        private string groupField;
        private string name;
        private  List<string> names;
        private string optionsField;

    private SubgroupRequestBuilder(string listId, string optionsField, string groupField, string name, List<String> names)
        {
            this.listId = listId;
            this.optionsField = optionsField;
            this.name = name;
            this.names = names;
            this.groupField = groupField;
        }
        public static SubgroupRequestBuilder getAddSubgroupRequestBuilder(string listId, string groupField, string name)
        {
            if (string.IsNullOrEmpty(listId)) { throw new System.ArgumentNullException("Parameter listId is null."); }
            if (string.IsNullOrEmpty(name)) { throw new System.ArgumentNullException("Parameter name is null."); }
            if (string.IsNullOrEmpty(groupField)) { throw new System.ArgumentNullException("Parameter groupField is null."); }

            return new SubgroupRequestBuilder(listId, null, groupField, name, null);
        }

        public static SubgroupRequestBuilder getAddSubgroupsRequestBuilder(String listId, String groupField, List<String> names)
        {
            if (string.IsNullOrEmpty(listId)) { throw new System.ArgumentNullException("Parameter listId is null."); }
            if (names == null) { throw new System.ArgumentNullException("Parameter names is null."); }
            if (string.IsNullOrEmpty(groupField)) { throw new System.ArgumentNullException("Parameter groupField is null."); }

            return new SubgroupRequestBuilder(listId, null, groupField, null, names);
        }
        public static SubgroupRequestBuilder getUpdateSubgroupRequestBuilder(String listId, String groupField, String optionField, String name)
        {
            if (string.IsNullOrEmpty(listId)) { throw new System.ArgumentNullException("Parameter listId is null."); }
            if (string.IsNullOrEmpty(name)) { throw new System.ArgumentNullException("Parameter name is null."); }
            if (string.IsNullOrEmpty(groupField)) { throw new System.ArgumentNullException("Parameter groupField is null."); }
            if (string.IsNullOrEmpty(optionField)) { throw new System.ArgumentNullException("Parameter optionField is null."); }

            return new SubgroupRequestBuilder(listId, optionField, groupField, name, null);
        }

        public SubgroupRequestBuilder.SubgroupRequest build()
        {
            if (name == null)
            {
                return new SubgroupRequestMultiple(this);
            }
            else if (names == null)
            {
                return new SubgroupRequestSingle(this);
            }
            else
            {
                Console.WriteLine("Unable to serialize Subgroup Request.");
                return null;
            }
        }

        [DataContract]
        public class SubgroupRequest : IJsonSerializable
        {
            [ScriptIgnore]
            private readonly  string listId;
            [ScriptIgnore]
            private readonly string groupField;
            [ScriptIgnore]
            private readonly string optionsField;

        public SubgroupRequest(SubgroupRequestBuilder builder)
        {
            this.listId = builder.listId;
            this.groupField = builder.groupField;
            this.optionsField = builder.optionsField;
        }

        public String getListId()
        {
            return listId;
        }

        public String getGroupField()
        {
            return groupField;
        }


        public String getOptionsField()
        {
            return optionsField;
        }


        public string getBody()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented,
                        new JsonSerializerSettings
                        {
                            NullValueHandling = NullValueHandling.Ignore
                        });
        }
        }

        //[DataContract]
        private  class SubgroupRequestSingle : SubgroupRequest
        {
            [DataMember(Name = "name")]
            private readonly string name;
            public SubgroupRequestSingle(SubgroupRequestBuilder builder) : base(builder)
            {
                this.name = builder.name;
            }

            public String getName()
            {
                return name;
            }
        }

        //[DataContract]
        private class SubgroupRequestMultiple : SubgroupRequest
        {
            [DataMember(Name = "name")]
            private  List<String> name;
            public SubgroupRequestMultiple(SubgroupRequestBuilder builder) : base(builder)
            {
                this.name = builder.names;
            }

            public List<String> getName()
            {
                return name;
            }
        }
    }
}
