﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using apifon_sdk_csharp.com.apifon.sdk.response;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    public class SubscriberFieldRequestBuilder : IHttpEtag
    {
        [ScriptIgnore]
        private readonly string listId;
        [ScriptIgnore]
        private string field;
        [ScriptIgnore]
        private string etag;
        private int length;
        private string name;
        private string type;
        private string dateFormat;
        private bool Internal;

        private SubscriberFieldRequestBuilder(string listId, string field)
        {
            this.field = field;
            this.listId = listId;
        }

        private SubscriberFieldRequestBuilder(string listId, string type, int length, string name)
        {
            this.type = type;
            this.length = length;
            this.name = name;
            this.Internal = false;
            this.listId = listId;
        }

        public static SubscriberFieldRequestBuilder getUpdateSubscriberFieldRequestBuilder(string listId, string field, string name, int length)
        {
            if (string.IsNullOrEmpty(field)) { throw new System.ArgumentNullException("Parameter field is null."); }
            return new SubscriberFieldRequestBuilder(listId, field).withName(name).withLength(length);
        }

        public static SubscriberFieldRequestBuilder getUpdateSubscriberFieldRequestBuilder(string listId, SubscriberFieldResponse subscriberField)
        {
            return new SubscriberFieldRequestBuilder(listId, subscriberField.getField())
                .withName(subscriberField.getName()).withLength(subscriberField.getLength())
                .withDateFormat(subscriberField.getDateFormat()).withInternal(subscriberField.isInternal());
        }

        public static SubscriberFieldRequestBuilder getAddRequestBuilder(string listId, string type, int length, string name)
        {
            return new SubscriberFieldRequestBuilder(listId, type, length, name);
        }

        public SubscriberFieldRequestBuilder withDateFormat(string dateFormat)
        {
            this.dateFormat = dateFormat;
            return this;
        }

        public SubscriberFieldRequestBuilder withInternal(bool Internal) 
        {
            this.Internal = Internal;
            return this;
        }

        public SubscriberFieldRequestBuilder withName(string name)
        {
            this.name = name;
            return this;
        }

        public SubscriberFieldRequestBuilder withLength(int length)
        {
            this.length = length;
            return this;
        }

        public SubscriberFieldRequestBuilder withListEtag(string listEtag)
        {
            this.etag = listEtag;
            return this;
        }

        public SubscriberFieldRequest build()
        {
            return new SubscriberFieldRequest(this);
        }

        public string getEtag()
        {
            return this.etag;
        }

        [DataContract]
        public class SubscriberFieldRequest : IJsonSerializable, IHttpEtag
        {
            [ScriptIgnore]
            private readonly string listId;
            [ScriptIgnore]
            private readonly string field;
            [ScriptIgnore]
            private readonly string etag;
            [DataMember(Name = "length")]
            private readonly int length;
            [DataMember(Name = "name")]
            private readonly string name;
            [DataMember(Name = "type")]
            private readonly string type;
            [DataMember(Name = "date_format")]
            private readonly string dateFormat;
            [DataMember(Name = "internal")]
            private readonly bool Internal;

            public SubscriberFieldRequest(SubscriberFieldRequestBuilder builder)
            {
                this.field = builder.field;
                this.length = builder.length;
                this.name = builder.name;
                this.type = builder.type;
                this.dateFormat = builder.dateFormat;
                this.Internal = builder.Internal;
                this.listId = builder.listId;
                this.etag = builder.etag;
            }

            public string getField()
            {
                return field;
            }

            public int getLength()
            {
                return length;
            }

            public string getName()
            {
                return name;
            }

            public string getType()
            {
                return type;
            }

            public string getDateFormat()
            {
                return dateFormat;
            }

            public bool isInternal()
            {
                return Internal;
            }

            public string getListId()
            {
                return listId;
            }

            public string getBody()
            {
                return JsonConvert.SerializeObject(this, Formatting.Indented,
                            new JsonSerializerSettings
                            {
                                NullValueHandling = NullValueHandling.Ignore
                            });
            }

            public string getEtag()
            {
                return this.etag;
            }
        }
    }
}


