﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    public class SubscriberQueryBuilder : PaginatedRequestBuilder, IFieldSerializable<SubscriberQueryBuilder>
    {
        private readonly string listId;
        private string status;
        private string emailStatus;
        private HashSet<string> fields;

        private SubscriberQueryBuilder(string listId)
        {
            this.listId = listId;
        }

        public static SubscriberQueryBuilder getQueryBuilder(string listId)
        {
            return new SubscriberQueryBuilder(listId);
        }

        public SubscriberQueryBuilder withStatus(string status)
        {
            this.status = status;
            return this;
        }

        public SubscriberQueryBuilder withEmailStatus(string emailStatus)
        {
            this.emailStatus = emailStatus;
            return this;
        }

        public override PaginatedRequest build()
        {
            return new SubscriberQueryRequest(this);
        }

        public override string toQueryParamRequest()
        {
            return build().toQueryParamRequest();
        }

        public HashSet<string> getFields()
        {
            return this.fields;
        }

        public override string getPathId()
        {
            return this.listId;
        }

        public SubscriberQueryBuilder addVisibleFields(Collection<string> fields)
        {
            if (fields == null) { throw new System.ArgumentNullException("Parameter fields is null."); }
            this.getFields().UnionWith(fields);
            return this;
        }

        public SubscriberQueryBuilder addVisibleField(string field)
        {
            if (field == null) { throw new System.ArgumentNullException("Parameter field is null."); }
            this.getFields().Add(field);
            return this;
        }

        public string getQueryParams()
        {
            throw new NotImplementedException();
        }

        public class SubscriberQueryRequest : PaginatedRequest
        {
            private readonly string listId;
            private readonly string status;
            private readonly string emailStatus;
            private readonly HashSet<string> fields;

            public SubscriberQueryRequest(SubscriberQueryBuilder builder)
            {
                this.listId = builder.listId;
                this.status = builder.status;
                this.emailStatus = builder.emailStatus;
                this.fields = builder.fields;
                this.page = builder.page;
                this.size = builder.size;
                this.sortDir = builder.sortDir;
                this.sortField = builder.sortField;
            }

            public override string getPathId()
            {
                return this.listId;
            }

            public override string toQueryParamRequest()
            {
                List<KeyValuePair<string, string>> queryParams = new List<KeyValuePair<string, string>>();
                addQueryParams(queryParams);
                if (!string.IsNullOrEmpty(status))
                {
                    queryParams.Add(new KeyValuePair<string, string>("status", status));
                }
                if (!string.IsNullOrEmpty(emailStatus))
                {
                    queryParams.Add(new KeyValuePair<string, string>("email_status", emailStatus));
                }
                if (fields != null && fields.Any(x => true))
                {
                    string join = string.Join(",", fields);
                    queryParams.Add(new KeyValuePair<string, string>("fields", join));
                }

                string joinParams = "";
                foreach (KeyValuePair<string, string> entry in queryParams)
                {
                    if (joinParams.Equals(""))
                    {
                        //For the first time
                        joinParams = entry.Key + "=" + entry.Value;
                    }
                    else
                    {
                        //Add "&" in query
                        joinParams = joinParams + "&" + entry.Key + "=" + entry.Value;
                    }
                }

                UriBuilder uriBuilder = new UriBuilder();
                uriBuilder.Query = joinParams;
                return uriBuilder.Query;
            }
        }
    }
}
