﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using apifon_sdk_csharp.com.apifon.sdk.response;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    public class SubscriberRequestBuilder : IHttpEtag
    {
        private readonly string listId;
        private readonly Dictionary<string, Object> subscriberFields;
        private readonly Dictionary<string, SubscriberGdprField> gdprFields;
        private string etag;
        private string destination;
        private string status;
        private string email;
        private string emailStatus;
        private string lookupMD5;
        private List<String> groupOptions;

        private SubscriberRequestBuilder(string listId)
        {
            this.listId = listId;
            this.subscriberFields = new Dictionary<string, Object>();
            this.gdprFields = new Dictionary<string, SubscriberGdprField>();
        }

        public static SubscriberRequestBuilder getBuilderFromSubscriber(SubscriberResponse subscriber)
        {
            return new SubscriberRequestBuilder(subscriber.getListId())
                    .withDestination(subscriber.getDestination(), subscriber.getStatus())
                    .withEmail(subscriber.getEmail(), subscriber.getEmailStatus())
                    .withSubscriberFields(subscriber.getSubscriberFields())
                    .withGdprFields(subscriber.getGdprFields())
                    .withLookupMD5(subscriber.getDestinationId() == null ? subscriber.getEmailId() : subscriber.getDestinationId());
        }

        public static SubscriberRequestBuilder getAddSubscriberRequestBuilder(string listId)
        {
            return new SubscriberRequestBuilder(listId);
        }

        public SubscriberRequestBuilder withLookupMD5(string lookupMD5)
        {
            this.lookupMD5 = lookupMD5;
            return this;
        }

        public SubscriberRequestBuilder withDestination(string destination, string status)
        {
            this.destination = destination;
            this.status = status;
            return this;
        }

        public SubscriberRequestBuilder withEmail(string email, string emailStatus)
        {
            this.email = email;
            this.emailStatus = emailStatus;
            return this;
        }

        public SubscriberRequestBuilder withStatus(string status)
        {
            this.status = status;
            return this;
        }

        public SubscriberRequestBuilder withEmailStatus(string emailStatus)
        {
            this.emailStatus = emailStatus;
            return this;
        }

        public SubscriberRequestBuilder withEtag(string etag)
        {
            this.etag = etag;
            return this;
        }

        public SubscriberRequestBuilder clearSubscriberFields()
        {
            this.subscriberFields.Clear();
            return this;
        }

        public SubscriberRequestBuilder withSubscriberField(string field, Object value)
        {
            this.subscriberFields.Add(field, value);
            return this;
        }

        public SubscriberRequestBuilder withSubscriberFields(Dictionary<string, Object> subscriberFields)
        {
            if(subscriberFields != null && subscriberFields.Count != 0)
            {
                foreach (var subscriberField in subscriberFields)
                    this.subscriberFields.Add(subscriberField.Key, subscriberField.Value);
            }
            return this;
        }

        public SubscriberRequestBuilder clearGdprFields()
        {
            this.gdprFields.Clear();
            return this;
        }

        public SubscriberRequestBuilder withGdprField(string field, bool gdprCompliance)
        {
            if (this.gdprFields.ContainsKey(field))
            {
                this.gdprFields[field].setMobile(gdprCompliance);
                this.gdprFields[field].setEmail(gdprCompliance);
            }
            else
            {
                SubscriberGdprField newGdprField = new SubscriberGdprField();
                newGdprField.setMobile(gdprCompliance);
                newGdprField.setEmail(gdprCompliance);
                this.gdprFields.Add(field, newGdprField);
            }
            return this;
        }

        public SubscriberRequestBuilder withMobileGdprField(string field, bool gdprCompliance)
        {
            if (this.gdprFields.ContainsKey(field))
            {
                this.gdprFields[field].setMobile(gdprCompliance);
            }
            else
            {
                SubscriberGdprField newGdprField = new SubscriberGdprField();
                newGdprField.setMobile(gdprCompliance);
                this.gdprFields.Add(field, newGdprField);
            }
            return this;
        }

        public SubscriberRequestBuilder withEmailGdprField(string field, bool gdprCompliance)
        {
            if (this.gdprFields.ContainsKey(field))
            {
                this.gdprFields[field].setEmail(gdprCompliance);
            }
            else
            {
                SubscriberGdprField newGdprField = new SubscriberGdprField();
                newGdprField.setEmail(gdprCompliance);
                this.gdprFields.Add(field, newGdprField);
            }
            return this;
        }

        public SubscriberRequestBuilder withGdprFields(Dictionary<string, SubscriberGdprField> gdprFields)
        {
            if (gdprFields != null && gdprFields.Count != 0)
            {
                foreach (var gdprField in gdprFields)
                    this.gdprFields.Add(gdprField.Key, gdprField.Value);
            }
            return this;
        }

        public SubscriberRequestBuilder withGroupOptions(List<String> groupOptions)
        {
            this.groupOptions = groupOptions;
            return this;
        }

        public SubscriberRequest build()
        {
            return new SubscriberRequest(this);
        }

        public string getEtag()
        {
            return this.etag;
        }

        [DataContract]
        public class SubscriberRequest : IJsonSerializable, IHttpEtag
        {
            [ScriptIgnore]
            private readonly string listId;
            [ScriptIgnore]
            private readonly string etag;
            [ScriptIgnore]
            private readonly string lookupMD5;
            [DataMember(Name = "destination")]
            private readonly string destination;
            [DataMember(Name = "status")]
            private string status;
            [DataMember(Name = "email")]
            private readonly string email;
            [DataMember(Name = "email_status")]
            private readonly string emailStatus;
            [DataMember(Name = "subscriber_fields")]
            private readonly Dictionary<string, Object> subscriberFields;
            [DataMember(Name = "gdpr_fields")]
            private readonly Dictionary<string, SubscriberGdprField> gdprFields;
            [DataMember(Name = "groupOptions")]
            private List<String> groupOptions;

            public SubscriberRequest(SubscriberRequestBuilder builder)
            {
                this.listId = builder.listId;
                this.etag = builder.etag;
                this.destination = builder.destination;
                this.status = builder.status;
                this.email = builder.email;
                this.emailStatus = builder.emailStatus;
                this.subscriberFields = builder.subscriberFields;
                this.gdprFields = builder.gdprFields;
                this.lookupMD5 = builder.lookupMD5;
                this.groupOptions = builder.groupOptions;
            }

            public string getListId()
            {
                return listId;
            }

            public string getDestination()
            {
                return destination;
            }

            public string getStatus()
            {
                return status;
            }

            public string getEmail()
            {
                return email;
            }

            public string getEmailStatus()
            {
                return emailStatus;
            }

            public Dictionary<string, Object> getSubscriberFields()
            {
                return subscriberFields;
            }

            public Dictionary<string, SubscriberGdprField> getGdprFields()
            {
                return gdprFields;
            }

            public string getLookupMD5()
            {
                return lookupMD5;
            }

            public string getEtag()
            {
                return this.etag;
            }

            public string getBody()
            {
                return JsonConvert.SerializeObject(this, Formatting.Indented,
                            new JsonSerializerSettings
                            {
                                NullValueHandling = NullValueHandling.Ignore
                            });
            }
        }
    }
}
