﻿using apifon_sdk_csharp.com.apifon.sdk.model;
using apifon_sdk_csharp.com.apifon.sdk.model.request;
using apifon_sdk_csharp.com.apifon.sdk.response;
using com.apifon.sdk.resource;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.sdk.resource
{
    class GroupResource : AbstractResource, UriLocator<GroupResponse>
    {
        public GroupResource() : base("/services/api/v1/list/{list_id}/group")
        {

        }

        public void replacePlaceholderAndSetUrl(string endpoint, string listId)
        {
            base.SetCurEndpoint(endpoint.Replace("{list_id}", listId));
        }

        public Uri addGroup(GroupRequestBuilder.GroupRequest request)
        {
            if (string.IsNullOrEmpty(request.getListId()))
            {
                throw new System.ArgumentException("request.listId is not provided.");
            }
            this.SetCurMethod("POST");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/", request.getListId());
            this.SetCurBody(request.getBody());

            WebResponse response = this.DispatchToResponse();
            return new Uri(response.Headers["Location"].ToString());
        }
        public Uri updateGroup(GroupRequestBuilder.GroupRequest request)
        {
            if (string.IsNullOrEmpty(request.getListId()))
            {
                throw new System.ArgumentException("request.listId is not provided.");
            }
            if (string.IsNullOrEmpty(request.getGroupField()))
            {
                throw new System.ArgumentException("request.groupField is not provided.");
            }
            this.SetCurMethod("PUT");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/" + request.getGroupField(), request.getListId());
            this.SetCurBody(request.getBody());

            WebResponse response = this.DispatchToResponse();
            return new Uri(response.Headers["Location"].ToString());
        }

        public GroupResponse getGroup(GroupSingleQuery request)
        {
            this.SetCurMethod("GET");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/" + request.getPathId(), request.getListId());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);

            return JsonConvert.DeserializeObject<GroupResponse>(responseContent);
        }

        public bool removeGroup(string listId, string groupField)
        {
            this.SetCurMethod("DELETE");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/" + groupField, listId);
            this.SetCurBody("");

            HttpWebResponse response = (HttpWebResponse)this.DispatchToResponse();
            return this.isSuccessful(response);
        }

        public GroupResponse getByResourceUri(Uri resourceUri)
        {
            this.SetCurMethod("GET");
            this.SetCurEndpoint(resourceUri.ToString());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);

            return JsonConvert.DeserializeObject<GroupResponse>(responseContent);
        }

        public CollectionEntityResponse<GroupResponse> getListByResouceUri(Uri resourceUri)
        {
            throw new NotImplementedException();
        }
    }
}
