﻿using apifon_sdk_csharp.com.apifon.sdk.exception;
using apifon_sdk_csharp.com.apifon.sdk.helpers;
using apifon_sdk_csharp.com.apifon.sdk.model;
using apifon_sdk_csharp.com.apifon.sdk.model.request;
using apifon_sdk_csharp.com.apifon.sdk.resource;
using apifon_sdk_csharp.com.apifon.sdk.response;
using com.apifon.sdk.model.request;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace com.apifon.sdk.resource
{
    public class ListResource : AbstractResource, UriLocator<ListResponse>
    {
        public ListResource() : base("/services/api/v1/list")
        {

        }

        public Uri createList(ListRequestBuilder.ListRequest request)
        {
            this.SetCurMethod("POST");
            this.SetCurEndpoint(this.GetEndpoint() + "/");
            this.SetCurBody(request.getBody());

            WebResponse response = this.DispatchToResponse();
            return new Uri(response.Headers["Location"].ToString());
        }

        public Uri updateList(ListRequestBuilder.ListRequest request)
        {
            if (string.IsNullOrEmpty(request.getId())) 
            {
                throw new System.ArgumentException("request.id is not set.");
            }
            this.SetCurMethod("PUT");
            this.SetCurEndpoint(this.GetEndpoint() + "/" + request.getId());
            this.SetCurBody(request.getBody());

            WebResponse response = this.DispatchToResponse();
            return new Uri(response.Headers["Content-Location"].ToString());
        }

        public ListResponse getList(ListSingleQuery request)
        {
            this.SetCurMethod("GET");
            this.SetCurEndpoint(this.GetEndpoint() + "/" + request.getPathId() + request.getQueryParams());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);
            
            return JsonConvert.DeserializeObject<ListResponse>(responseContent); 
        }

        public CollectionEntityResponse<ListResponse> getLists(PaginatedRequest request)
        {
            this.SetCurMethod("GET");
            this.SetCurEndpoint(this.GetEndpoint() + "/" + request.toQueryParamRequest());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);
            
            return JsonConvert.DeserializeObject<CollectionEntityResponse<ListResponse>>(responseContent);
        }

        public CollectionEntityResponse<ListResponse> getLists(String paginationUrl)
        {
            return getListByResouceUri(new Uri(paginationUrl));
        }

        public bool removeList(string id)
        {
            this.SetCurMethod("DELETE");
            this.SetCurEndpoint(this.GetEndpoint() + "/" + id);
            this.SetCurBody("");

            HttpWebResponse response = (HttpWebResponse)this.DispatchToResponse();
            return this.isSuccessful(response);
        }

        public ListResponse getByResourceUri(Uri resourceUri)
        {
            string[] pathParams = resourceUri.Segments;
            string listId = pathParams[pathParams.Length - 1];
            return getList(ListSingleQuery.getListRequest(listId));
        }

        public CollectionEntityResponse<ListResponse> getListByResouceUri(Uri resourceUri)
        {
            this.SetCurMethod("GET");
            this.SetCurEndpoint(resourceUri.ToString());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);
            
            return JsonConvert.DeserializeObject<CollectionEntityResponse<ListResponse>>(responseContent);
        }
    }
}
