﻿using apifon_sdk_csharp.com.apifon.sdk.model;
using apifon_sdk_csharp.com.apifon.sdk.model.request;
using apifon_sdk_csharp.com.apifon.sdk.response;
using com.apifon.sdk.resource;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.sdk.resource
{
    class SubgroupResource : AbstractResource, UriLocator<SubgroupResponse>
    {
        public SubgroupResource() : base("/services/api/v1/list/{list_id}/{group_field}")
        {

        }

        public void replacePlaceholderAndSetUrl(string endpoint, string listId,string groupField)
        {
            endpoint = endpoint.Replace("{list_id}", listId);
            base.SetCurEndpoint(endpoint.Replace("{group_field}", groupField));
        }


        public Uri addSubgroup(SubgroupRequestBuilder.SubgroupRequest request)
        {
            if (string.IsNullOrEmpty(request.getListId()))
            {
                throw new System.ArgumentException("request.listId is not provided.");
            }
            if (string.IsNullOrEmpty(request.getGroupField()))
            {
                throw new System.ArgumentException("request.groupField is not provided.");
            }
            this.SetCurMethod("POST");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/option", request.getListId(),request.getGroupField());
            this.SetCurBody(request.getBody());

            WebResponse response = this.DispatchToResponse();
            return new Uri(response.Headers["Location"].ToString());
        }

        public Uri addSubgroups(SubgroupRequestBuilder.SubgroupRequest request)
        {
            if (string.IsNullOrEmpty(request.getListId()))
            {
                throw new System.ArgumentException("request.listId is not provided.");
            }
            if (string.IsNullOrEmpty(request.getGroupField()))
            {
                throw new System.ArgumentException("request.groupField is not provided.");
            }
            this.SetCurMethod("POST");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/options", request.getListId(), request.getGroupField());
            this.SetCurBody(request.getBody());

            WebResponse response = this.DispatchToResponse();
            return new Uri(response.Headers["Location"].ToString());
        }


        public Uri updateSubgroup(SubgroupRequestBuilder.SubgroupRequest request)
        {
            if (string.IsNullOrEmpty(request.getListId()))
            {
                throw new System.ArgumentException("request.listId is not provided.");
            }
            if (string.IsNullOrEmpty(request.getGroupField()))
            {
                throw new System.ArgumentException("request.groupField is not provided.");
            }
            if (string.IsNullOrEmpty(request.getOptionsField()))
            {
                throw new System.ArgumentException("request.optionsField is not provided.");
            }
            this.SetCurMethod("PUT");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/" + request.getOptionsField(), request.getListId(),request.getGroupField());
            this.SetCurBody(request.getBody());

            WebResponse response = this.DispatchToResponse();
            return new Uri(response.Headers["Location"].ToString());
        }


        public SubgroupResponse getSubgroup(SubgroupSingleQuery request)
        {
            this.SetCurMethod("GET");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/option/" + request.getPathId(), request.getListId(),request.getGroupField());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);

            return JsonConvert.DeserializeObject<SubgroupResponse>(responseContent);
        }

        public bool removeSubgroup(string listId, string groupField, string optionsField)
        {
            this.SetCurMethod("DELETE");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/" + optionsField, listId,groupField);
            this.SetCurBody("");

            HttpWebResponse response = (HttpWebResponse)this.DispatchToResponse();
            return this.isSuccessful(response);
        }


        public SubgroupResponse getByResourceUri(Uri resourceUri)
        {
            this.SetCurMethod("GET");
            this.SetCurEndpoint(resourceUri.ToString());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);

            return JsonConvert.DeserializeObject<SubgroupResponse>(responseContent);
        }

        public CollectionEntityResponse<SubgroupResponse> getListByResouceUri(Uri resourceUri)
        {
            throw new NotImplementedException();
        }
    }
}
