﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using apifon_sdk_csharp.com.apifon.sdk.model;
using apifon_sdk_csharp.com.apifon.sdk.model.request;
using apifon_sdk_csharp.com.apifon.sdk.response;
using com.apifon.sdk.resource;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using static apifon_sdk_csharp.com.apifon.sdk.model.request.SubscriberConditionQueryRequestBuilder;

namespace apifon_sdk_csharp.com.apifon.sdk.resource
{
    public class SubscriberResource : AbstractResource, UriLocator<SubscriberResponse>
    {
        public SubscriberResource() : base("/services/api/v1/list/{list_id}/subscriber")
        {

        }

        public void replacePlaceholderAndSetUrl(string endpoint, string listId)
        {
            base.SetCurEndpoint(endpoint.Replace("{list_id}", listId));
        }

        public Uri addSubscriber(SubscriberRequestBuilder.SubscriberRequest request)
        {
            if (string.IsNullOrEmpty(request.getListId()))
            {
                throw new System.ArgumentException("request.listId is not provided.");
            }
            this.SetCurMethod("POST");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/", request.getListId());
            this.SetCurBody(request.getBody());

            WebResponse response = this.DispatchToResponse();
            return new Uri(response.Headers["Location"].ToString());
        }

        public Uri updateSubscriber(SubscriberRequestBuilder.SubscriberRequest request)
        {
            if (string.IsNullOrEmpty(request.getListId()) || string.IsNullOrEmpty(request.getLookupMD5())) 
            {
                throw new System.ArgumentException("request.listId or request.lookupMD5 is not provided.");
            }
            this.SetCurMethod("PUT");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/" + request.getLookupMD5(), request.getListId());
            this.SetCurBody(request.getBody());

            WebResponse response = this.DispatchToResponse();
            return new Uri(response.Headers["Content-Location"].ToString());
        }

        public SubscriberResponse getSubscriber(SubscriberSingleQuery request)
        {
            this.SetCurMethod("GET");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/" + request.getPathId() + request.getQueryParams(), request.getListId());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);

            return JsonConvert.DeserializeObject<SubscriberResponse>(responseContent);
        }

        public CollectionEntityResponse<SubscriberResponse> querySubscribersWithConditions(SubscriberConditionQueryRequest request)
        {
            this.SetCurMethod("POST");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/query", request.getListId());
            this.SetCurBody(request.getBody());

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);
            
            return JsonConvert.DeserializeObject<CollectionEntityResponse<SubscriberResponse>>(responseContent);
        }

        public CollectionEntityResponse<SubscriberResponse> querySubscribers(PaginatedRequest request)
        {
            this.SetCurMethod("GET");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + request.toQueryParamRequest(), request.getPathId());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);
            
            return JsonConvert.DeserializeObject<CollectionEntityResponse<SubscriberResponse>>(responseContent);
        }

        public CollectionEntityResponse<SubscriberResponse> querySubscribers(string paginationUrl)
        {
            return getListByResouceUri(new Uri(paginationUrl));
        }

        public bool subscribeSubscriber(string listId, string lookupMD5)
        {
            this.SetCurMethod("POST");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/" + lookupMD5 + "/subscribe", listId);
            this.SetCurBody("");

            HttpWebResponse response = (HttpWebResponse)this.DispatchToResponse();
            return this.isSuccessful(response);
        }

        public bool unsubscribeSubscriber(string listId, string lookupMD5)
        {
            this.SetCurMethod("POST");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/" + lookupMD5 + "/unsubscribe", listId);
            this.SetCurBody("");

            HttpWebResponse response = (HttpWebResponse)this.DispatchToResponse();
            return this.isSuccessful(response);
        }

        public bool removeSubscriber(string listId, string lookupMD5)
        {
            this.SetCurMethod("DELETE");
            this.replacePlaceholderAndSetUrl(this.GetEndpoint() + "/" + lookupMD5, listId);
            this.SetCurBody("");

            HttpWebResponse response = (HttpWebResponse)this.DispatchToResponse();
            return this.isSuccessful(response);
        }

        public SubscriberResponse getByResourceUri(Uri resourceUri)
        {
            this.SetCurMethod("GET");
            this.SetCurEndpoint(resourceUri.ToString());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);
            
            return JsonConvert.DeserializeObject<SubscriberResponse>(responseContent);
        }

        public CollectionEntityResponse<SubscriberResponse> getListByResouceUri(Uri resourceUri)
        {
            this.SetCurMethod("GET");
            this.SetCurEndpoint(resourceUri.ToString());
            this.SetCurBody("");

            WebResponse response = this.DispatchToResponse();
            string responseContent = this.getResponseBody(response);
            
            return JsonConvert.DeserializeObject<CollectionEntityResponse<SubscriberResponse>>(responseContent);
        }
    }
}
