﻿using apifon_sdk_csharp.com.apifon.sdk.exception;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.sdk.resource.helpers
{
    [DataContract]
    public class OAuthCredentials :ICloneable
    {
        [DataMember(Name = "access_token")]
        private String accessToken;
        [DataMember(Name = "refresh_token")]
        private String refreshToken;
        [DataMember(Name = "access_code")]
        private String accessCode;
        [DataMember(Name = "expires_in")]
        private long expiresIn;
        private DateTime expirationDate;

        public static OAuthCredentials getOAuthCredentialsFromAccessCode(string accessCode)
        {
            OAuthCredentials credentials = new OAuthCredentials();
            credentials.setAccessCode(accessCode);
            return credentials;
        }

        public OAuthCredentials()
        {

        }

        public string getAccessToken()
        {
            return accessToken;
        }

        public void setAccessToken(string accessToken)
        {
            this.accessToken = accessToken;
        }

        public string getRefreshToken()
        {
            return refreshToken;
        }

        public void setRefreshToken(string refreshToken)
        {
            this.refreshToken = refreshToken;
        }

        public string getAccessCode()
        {
            return accessCode;
        }

        public void setAccessCode(string accessCode)
        {
            this.accessCode = accessCode;
        }

        public long getExpiresIn()
        {
            return expiresIn;
        }

        public void setExpiresIn(long expiresIn)
        {
            this.expiresIn = expiresIn;
            try 
            {
                this.expirationDate = DateTime.UtcNow.AddMilliseconds(expiresIn * 1000);

            }
            catch (Exception ex) 
            {
                throw new OAuthTokenException("Failed to parsed expires in from value " + expiresIn);
            }
        }

        public DateTime getExpirationDate()
        {
            return expirationDate;
        }

        public void setExpirationDate(DateTime expirationDate)
        {
            this.expirationDate = expirationDate;
        }

        public bool hasExpired()
        {
            if (this.expirationDate != null)
            {
                return DateTime.UtcNow > (this.expirationDate);
            }
            return true;
        }

        public object Clone()
        {
            base.MemberwiseClone();
            OAuthCredentials copy = new OAuthCredentials();
            copy.accessCode = this.accessCode;
            copy.accessToken = this.accessToken;
            copy.expiresIn = this.expiresIn;
            copy.refreshToken = this.refreshToken;
            copy.expirationDate = this.expirationDate;
            return copy;
        }

        public override string ToString()
        {
            return "OAuthCredentials{" + "accessToken=" + accessToken + ", refreshToken=" + refreshToken + ", accessCode=" + accessCode + ", expiresIn=" + expiresIn + ", expirationDate=" + expirationDate + '}';
        }
    }
}
